#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

from ..api.gitlab_api_access import *
from ..api.gitlab_api_queries import *
from ..utils.option import *
from ..arguments.arguments import *


class Migration:

    def __init__(self, arguments):
        self._help = False
        _gitlab_api_url_src = None
        _token_src = None
        _project_src = None
        _gitlab_api_url_target = None
        _token_target = None
        _project_target = None
        for i in range(len(arguments)):
            if option(arguments[i], gitlab_api_url_src):
                _gitlab_api_url_src = arguments[i + 1]
                i += 1
            elif option(arguments[i], gitlab_api_url_target):
                _gitlab_api_url_target = arguments[i + 1]
                i += 1
            if option(arguments[i], token_src):
                _token_src = arguments[i + 1]
                i += 1
            elif option(arguments[i], token_target):
                _token_target = arguments[i + 1]
                i += 1
            elif option(arguments[i], project_src):
                _project_src = arguments[i + 1]
                i += 1
            elif option(arguments[i], project_target):
                _project_target = arguments[i + 1]
                i += 1
            elif option(arguments[i], help_short, help_long):
                self._help = True
        self._access_src = GitLabAPIAccess(_gitlab_api_url_src, _token_src, _project_src)
        self._access_target = GitLabAPIAccess(_gitlab_api_url_target, _token_target, _project_target)
        self._queries_src = GitLabAPIQueries(self._access_src)
        self._queries_target = GitLabAPIQueries(self._access_target)

    def help(self, gazix="", program=""):
        print(gazix + " " + program + " [options]")
        print("\toptions:")
        print("\t--help            Print the help")
        print("\t--gitlab_src      Gitlab API url of the source repository")
        print("\t--gitlab_target   Gitlab API url of the target repository")
        print("\t--token_src       Access token of the source repository")
        print("\t--token_target    Access token of the target repository")
        print("\t--project_src     ID of the source repository")
        print("\t--project_target  ID token of the target repository")



#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

# -*- coding: utf-8 -*-
import logging

from ..utils.command import *
from ..utils.json import *
from ..utils.log import *
from .gitlab_api_object import *
import urllib.parse


def curl2json(line):
    data = command(line, output=True)
    data = read_json(data)
    return data


def get_jobs(host, project, token, filters={}):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/jobs'
    line += "?"
    for filter in filters:
        line += filter + "=" + str(filters[filter]) + "&"
    line = line[:-1]
    line += '"'
    return curl2json(line)


def get_pipeline_jobs(host, project, token, pipeline, filters={}):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/pipelines/' + str(pipeline) + '/jobs'
    line += "?"
    for filter in filters:
        line += filter + "=" + str(filters[filter]) + "&"
    line = line[:-1]
    line += '"'
    return curl2json(line)


def get_active_jobs(host, project, token):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/jobs?scope[]=pending&scope[]=running"'
    return curl2json(line)





def get_pipeline(host, project, token, pipeline):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/pipelines/' + str(pipeline) + '"'
    return curl2json(line)


def get_pipeline_variable(host, project, token, pipeline):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/pipelines/' + str(pipeline) + '/variables"'
    return curl2json(line)


def terminated(status):
    return status in ["success", "failed", "canceled", "skipped", "manual"]


def pipeline_terminated(host, project, token, pipeline):
    data = get_pipeline(host, project, token, pipeline)
    status = data["status"]
    return terminated(status)


def pipeline_success(host, project, token, pipeline):
    data = get_pipeline(host, project, token, pipeline)
    status = data["status"]
    return status in ["success"]


def get_pipelines(host, project, token, filters={}):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/pipelines'
    line += "?"
    for filter in filters:
        line += filter + "=" + str(filters[filter]) + "&"
    line = line[:-1]
    line += '"'
    return curl2json(line)


def kill_pipeline(host, project, token, pipeline):
    line = "curl -s --request POST --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/pipelines/' + str(pipeline) + '/cancel"'
    return command(line)


def get_merge_request(host, project, token, merge_request_iid, options=None):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid)
    if options is not None:
        line += "?" + options
    line += '"'
    return curl2json(line)


def update_merge_request(host, project, token, merge_request_iid, options=None):
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid)
    if options is not None:
        line += "?" + options
    line += '"'
    return curl2json(line)


def close_merge_request(host, project, token, merge_request_iid):
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid) + '?state_event=close"'
    return curl2json(line)


def cancel_merge_when_pipeline_succeeds(host, project, token, merge_request_iid):
    line = "curl -s --request POST --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(
        merge_request_iid) + "/cancel_merge_when_pipeline_succeeds"
    line += '"'
    return curl2json(line)


def update_merge_request_merge(host, project, token, merge_request_iid, options=None):
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid) + "/merge"
    if options is not None:
        line += "?" + options
    line += '"'
    return curl2json(line)


def close_issue(host, project, token, iid):
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/issues/' + str(iid) + '?state_event=close"'
    return curl2json(line)


def add_label_to_merge_request(host, project, token, merge_request_iid, old_labels, label):
    data = ""
    for lab in old_labels:
        data += lab + ","
    data += label
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += ' --data "labels=' + data + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid) + '"'
    return curl2json(line)


def set_labels_to_merge_request(host, project, token, merge_request_iid, labels):
    data = ""
    for lab in labels:
        data += lab + ","
    data = data[:-1]
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += ' --data "labels=' + data + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid) + '"'
    return curl2json(line)


def add_label_to_issue(host, project, token, iid, old_labels, label):
    data = ""
    for lab in old_labels:
        data += lab + ","
    data += label
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += ' --data "labels=' + data + '"'
    line += " " + '"' + host + '/' + project + '/issues/' + str(iid) + '"'
    return curl2json(line)


def set_labels_to_issue(host, project, token, iid, labels):
    data = ""
    for lab in labels:
        data += lab + ","
    data = data[:-1]
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += ' --data "labels=' + data + '"'
    line += " " + '"' + host + '/' + project + '/issues/' + str(iid) + '"'
    return curl2json(line)


def get_opened_merge_requests(host, project, token):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests?scope=all&order_by=updated_at&sort=asc&state=opened' + '"'
    return curl2json(line)


def get_ready_merge_requests(host, project, token):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests?scope=all&order_by=updated_at&sort=asc&state=opened&wip=no&target_branch=master' + '"'
    return curl2json(line)


def get_merged_merge_requests(host, project, token):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests?scope=all&order_by=updated_at&sort=desc&state=merged' + '"'
    return curl2json(line)


def get_opened_issues(host, project, token):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/issues?scope=all&order_by=updated_at&sort=asc&state=opened' + '"'
    return curl2json(line)


def get_pipelines_from_merge_request(host, project, token, merge_request_iid):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid) + '/pipelines"'
    return curl2json(line)


def comment_merge_request(host, project, token, merge_request_iid, message):
    line = "curl --request POST --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(
        merge_request_iid) + '/notes?body=' + urllib.parse.quote(message) + '"'
    return command(line)


def comment_issue(host, project, token, iid, message):
    line = "curl --request POST --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/issues/' + str(iid) + '/notes?body=' + urllib.parse.quote(
        message) + '"'
    return command(line)


def rebase_merge_request(host, project, token, merge_request_iid):
    line = "curl -s --request PUT --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request_iid) + '/rebase"'
    return curl2json(line)


def is_merge_request_rebased(host, project, token, merge_request_iid):
    return get_merge_request(host, project, token, merge_request_iid, options="include_rebase_in_progress=true")


def download_job_artifacts(host, project, token, job_id, archive):
    line = "curl -s -L --output " + archive
    line += " --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/jobs/' + str(job_id) + '/artifacts"'
    return command(line)


def get_merge_request_approvals(host, project, token, merge_request):
    line = "curl -s --globoff --header "
    line += '"PRIVATE-TOKEN: ' + token + '"'
    line += " " + '"' + host + '/' + project + '/merge_requests/' + str(merge_request) + '/approvals"'
    print(line)
    return curl2json(line)


##


def is_pipeline_terminated(pipeline):
    return pipeline["status"] in ["success", "failed", "canceled", "skipped", "manual"]


class GitLabAPIQueries:

    def __init__(self, access):
        self._access = access

    def GET(self, query, query2=None):
        line = "curl -s --globoff --header "
        line += '"PRIVATE-TOKEN: ' + self._access.get_token() + '"'
        line += " " + '"' + self._access.get_url()
        line += '/projects/' + self._access.get_project() + "/"
        if isinstance(query, str):
            line += query
        if isinstance(query, GitLabAPIObject):
            line += query.get_type() + "/" + query.get_query()
        if query2 is not None:
            if isinstance(query2, str):
                line += "/" + query2
        line += '"'
        return line

    def PUT(self, query, data=None):
        logging.debug(query)
        line = "curl -s --header "
        line += '"PRIVATE-TOKEN: ' + self._access.get_token() + '"'
        if data is not None:
            line += ' --data "' + data + '"'
        line += " " + '"' + self._access.get_url()
        line += '/projects/' + self._access.get_project() + "/" + query + '"'
        return line

    def POST(self, query, data=None, query2=None):
        logging.debug(query)
        line = "curl -s --request POST --header "
        line += '"PRIVATE-TOKEN: ' + self._access.get_token() + '"'
        if data is not None:
            line += ' --data "' + data + '"'
        line += " " + '"' + self._access.get_url()
        line += '/projects/' + self._access.get_project() + "/"
        if isinstance(query, str):
            line += query
        if isinstance(query, GitLabAPIObject):
            line += query.get_type() + "/" + query.get_query()
        if query2 is not None:
            if isinstance(query2, str):
                line += "/" + query2
        line += '"'
        return line

    def list_labels(self):
        return curl2json(self.GET("labels"))

    def create(self, obj):
        return curl2json(self.PUT(obj.get_type(), obj.get_url()))

    def trigger_pipeline(self, reference, variables):
        line = "curl -s -X POST"
        line += " -F token=" + self._access.get_token()
        line += ' -F "ref=' + reference + '"'
        for var in variables:
            line += ' -F "variables[' + var + ']=' + variables[var] + '"'
            line += " " + '"' + self._access.get_url()
            line += '/projects/' + self._access.get_project()
            line += '/trigger/pipeline"'
        return curl2json(line)

    def is_pipeline_terminated(self, pipeline):
        return is_pipeline_terminated(self.GET(pipeline))




#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

from ..api.gitlab_api_access import *
from ..api.gitlab_api_queries import *
from ..arguments.arguments import *
from ..utils.option import *
from ..utils.log import *
from ..api.gitlab_api_pipeline import *

labels_program = "trigger"

_range = 3
_sleep = 60

_range_unstable = 5
_sleep_unstable = 300


class Trigger:

    def __init__(self, arguments):
        self._help = False
        _gitlab_api_url = None
        _token_trigger = None
        _project = None
        self._variables = {}
        self._reference = None
        self._wait = False
        self._unstable = False
        for i in range(len(arguments)):
            if option(arguments[i], gitlab_api_url):
                _gitlab_api_url = arguments[i + 1]
                i += 1
            if option(arguments[i], token_trigger):
                _token_trigger = arguments[i + 1]
                i += 1
            elif option(arguments[i], project):
                _project = arguments[i + 1]
                i += 1
            elif option(arguments[i], variable_short, variable_long):
                key, value = arguments[i + 1].split(":", 1)
                self._variables[key] = value
                i += 1
            elif option(arguments[i], reference_short, reference_long):
                self._reference = arguments[i + 1]
                i += 1
            elif option(arguments[i], wait_short, wait_long):
                self._wait = True
            elif option(arguments[i], unstable_short, unstable_long):
                self._unstable = True
            elif option(arguments[i], help_short, help_long):
                self._help = True
        self._access = GitLabAPIAccess(_gitlab_api_url, _token_trigger, _project)
        self._queries = GitLabAPIQueries(self._access)
        if self._help:
            self.help(gazix, labels_program)
            exit(0)
        if not self._access.is_valid():
            self.help(gazix, labels_program)
            exit(1)
        exit(self.trigger())

    def trigger(self):
        pipeline = self._queries.trigger_pipeline(self._reference, self._variables)
        pipeline_object = GitlLabAPIPipeline(pipeline["id"])
        logger.info("Pipeline triggered... Link: " + pipeline["web_url"])

        r = _range
        s = _sleep

        if self._unstable:
            r = _range_unstable
            s = _sleep_unstable

        if self._wait:
            term = False
            while not term:
                print(".", end="")
                time.sleep(s)
                for i in range(0, r):
                    try:
                        term = self._queries.is_pipeline_terminated(pipeline_object)
                    except:
                        print()
                        print("Impossible to join the pipeline!")
                        if i == 2:
                            print("Connexion failure!")
                            return 2
                        print("Trying again...!")
                        time.sleep(s)
                    else:
                        break
        return 0

    def help(self, gazix="", program=""):
        print(gazix + " " + program + " [options]")
        print("\toptions:")
        print("\t-h/--help                Print the help")
        print("\t--gitlab [value]         Gitlab API url of the repository")
        print("\t--token_trigger [value]  Trigger token of the repository")
        print("\t--project [value]        ID of the repository")
        print("\t-v/--var [key]:[value]   Variable to transmit to the pipeline")
        print("\t-r/--ref [value]         Reference of the repository (main, master, etc.)")
        print("\t-w/--wait                Wait for the end of the pipeline execution")
        print("\t-u/--unstable            Unstable mode")



#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

import logging

from .migration import *
from ..arguments.arguments import *
from ..api.gitlab_api_label import *
from ..utils.log import *

labels_program = "cp-labels"


def list_labels(queries):
    label_dict = queries.list_labels()
    logging.debug(label_dict)
    labels = {}
    for lab in label_dict:
        name = lab["name"]
        color = lab["color"]
        description = lab["description"]
        priority = lab["priority"]
        labels[name] = GitlLabAPILabel(name, color, description, priority)
        logger.debug(labels[name])
    return labels


class Labels(Migration):

    def __init__(self, arguments):
        super().__init__(arguments)
        if self._help:
            self.help(gazix, labels_program)
            exit(0)
        if not self._access_src.is_valid() or not self._access_target.is_valid():
            self.help(gazix, labels_program)
            exit(1)
        self.add_missing_labels()

    def add_missing_labels(self):
        labels_src = list_labels(self._queries_src)
        labels_target = list_labels(self._queries_target)
        for lab in labels_src:
            if lab not in labels_target:
                logger.debug(lab + " not in target")
                json = self._queries_target.create(labels_src[lab])
                logger.debug(json)
            else:
                logger.debug(lab + " in target")






#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

import sys
from .utils.option import option
from .migration.labels import Labels
from .trigger.trigger import Trigger
from .kill.kill_old import KillOld

def help():
    print("gazix [option]")
    print("\toption:")
    print("\thelp          Print the help")
    #print("\tcp-MR         Copy all the merge requests from a source to a target repositories")
    #print("\tcp-issues     Copy all the issues from a source to a target repositories")
    print("\tcp-labels     Copy all the labels from a source to a target repositories")
    print("\ttrigger       Trigger the pipeline of a target repository")
    print("\tkill-old      Kill the old running pipelines of a MR")


def main():
    program = None
    try:
        program = sys.argv[1]
        status = "help" in program
        if status:
            raise Exception
    except:
        help()
        exit()
    try:
        args = sys.argv[2:]
    except:
        args = None
    if option(program, "cp-labels"):
        Labels(args)
    if option(program, "trigger"):
        Trigger(args)
    if option(program, "kill-old"):
        KillOld(args)


if __name__ == "__main__":
    main()



#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

# -*- coding: utf-8 -*-
import os
import subprocess as subp
import signal


def command(line, stdout=False, output=False, timeout=1200):
    """
        Execute a shell command
    """
    if output:
        stdout = True

    if not stdout:
        line += ""

    if output:
        line += ""
        output = ""
        try:
            p = subp.Popen(line,
                           stdout=subp.PIPE,
                           stderr=subp.STDOUT,
                           shell=True)

            output, err = p.communicate(timeout=timeout)
        except subp.TimeoutExpired:
            os.kill(p.pid, signal.SIGTERM)
            output, err = p.communicate()
            return output.decode('utf-8')+"\nEXECUTION TIME OUT"
        else:
            return output.decode('utf-8')
    try:
        p = subp.Popen(line, stdout=subp.PIPE, shell=True, stderr=subp.PIPE)
        output, err = p.communicate(timeout=timeout)
    except subp.TimeoutExpired:
        os.kill(p.pid, signal.SIGTERM)
        return False
    else:
        if p.returncode != 0:
            return False
    return True


def critical_command(line):
    err = subp.call([line+" > /dev/null 2>&1 "], shell=True)
    if err:
        raise RuntimeError("An error has been detected during the " +
                           "execution of: " + line)
    return True




#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

gitlab_api_url = "--gitlab"
gitlab_api_url_src = "--gitlab_src"
gitlab_api_url_target = "--gitlab_target"

token = "--token"
token_src = "--token_src"
token_target = "--token_target"
token_trigger = "--token_trigger"

project = "--project"
project_src = "--project_src"
project_target = "--project_target"

MR = "--MR"

pipeline = "--pipeline"

variable_long = "--var"
variable_short = "-v"

reference_long = "--ref"
reference_short = "-r"

wait_long = "--wait"
wait_short = "-w"

help_short = "-h"
help_long = "--help"

unstable_short = "-u"
unstable_long = "--unstable"

gazix = "gazix"
